using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Addresses.Commands.DeleteAddress;

public class DeleteAddressCommandAuthorizer :
    AbstractRequestAuthorizer<DeleteAddressCommand>
{
    private readonly SessionUserService _sessionUserService;

    public DeleteAddressCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(DeleteAddressCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
