using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Addresses.Queries.GetAddress;

public class GetAddressQueryHandler :
    IRequestHandler<GetAddressQuery, AddressDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetAddressQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<AddressDto> Handle(
        GetAddressQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.AddressRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.City.Region.Country,
            cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<AddressDto>(entity);
    }
}
