using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Models;
using cuqmbr.TravelGuide.Application.Common.Extensions;

namespace cuqmbr.TravelGuide.Application.Addresses.Queries.GetAddressesPage;

public class GetAddressesPageQueryHandler :
    IRequestHandler<GetAddressesPageQuery, PaginatedList<AddressDto>>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetAddressesPageQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<PaginatedList<AddressDto>> Handle(
        GetAddressesPageQuery request,
        CancellationToken cancellationToken)
    {
        var paginatedList = await _unitOfWork.AddressRepository.GetPageAsync(
            e =>
                (e.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.City.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.City.Region.Name.ToLower().Contains(request.Search.ToLower()) ||
                e.City.Region.Country.Name.ToLower().Contains(request.Search.ToLower())) &&
                (request.LongitudeGreaterOrEqualThan != null
                    ? e.Longitude >= request.LongitudeGreaterOrEqualThan
                    : true) &&
                (request.LongitudeLessOrEqualThan != null
                    ? e.Longitude <= request.LongitudeLessOrEqualThan
                    : true) &&
                (request.LatitudeGreaterOrEqualThan != null
                    ? e.Latitude >= request.LatitudeGreaterOrEqualThan
                    : true) &&
                (request.LatitudeLessOrEqualThan != null
                    ? e.Latitude <= request.LatitudeLessOrEqualThan
                    : true) &&
                (request.VehicleType != null
                    ? e.VehicleType == request.VehicleType
                    : true) &&
                (request.CityGuid != null
                    ? e.City.Guid == request.CityGuid
                    : true) &&
                (request.RegionGuid != null
                    ? e.City.Region.Guid == request.RegionGuid
                    : true) &&
                (request.CountryGuid != null
                    ? e.City.Region.Country.Guid == request.CountryGuid
                    : true),
            e => e.City.Region.Country,
            request.PageNumber, request.PageSize,
            cancellationToken);

        var mappedItems = _mapper
            .ProjectTo<AddressDto>(paginatedList.Items.AsQueryable());

        mappedItems = QueryableExtension<AddressDto>
            .ApplySort(mappedItems, request.Sort);

        _unitOfWork.Dispose();

        return new PaginatedList<AddressDto>(
            mappedItems.ToList(),
            paginatedList.TotalCount, request.PageNumber,
            request.PageSize);
    }
}
