using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Aircrafts.Queries.GetAircraft;

public class GetAircraftQueryHandler :
    IRequestHandler<GetAircraftQuery, AircraftDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetAircraftQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<AircraftDto> Handle(
        GetAircraftQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.AircraftRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.Company,
            cancellationToken);

        _unitOfWork.Dispose();

        if (entity == null)
        {
            throw new NotFoundException();
        }

        return _mapper.Map<AircraftDto>(entity);
    }
}
