using cuqmbr.TravelGuide.Application.Common.Exceptions;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using MediatR;

namespace cuqmbr.TravelGuide.Application.Authenticaion.Commands.RevokeRefreshToken;

public class RevokeRefreshTokenCommandHandler :
    IRequestHandler<RevokeRefreshTokenCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public RevokeRefreshTokenCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        RevokeRefreshTokenCommand request, CancellationToken cancellationToken)
    {
        var refreshToken = (await _unitOfWork.RefreshTokenRepository
            .GetOneAsync(e => e.Value == request.RefreshToken,
                cancellationToken));

        if (refreshToken == null)
        {
            throw new AuthenticationException("Invalid refreshToken");
        }

        if (!refreshToken.IsActive)
        {
            throw new AuthenticationException("RefreshToken already revoked");
        }

        refreshToken.RevocationTime = DateTimeOffset.UtcNow;

        await _unitOfWork.RefreshTokenRepository
            .UpdateOneAsync(refreshToken, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
