using System.Linq.Dynamic.Core;
using System.Reflection;
using System.Text;

namespace cuqmbr.TravelGuide.Application.Common.Extensions;

public static class QueryableExtension<T>
{
    public static IQueryable<T> ApplySort(
        IQueryable<T> entities, string? orderQueryString)
    {
        if (!entities.Any() || String.IsNullOrWhiteSpace(orderQueryString))
        {
            return entities;
        }


        var orderParameters = orderQueryString.Trim().Split(",");


        var queryPropertyTypes = typeof(T)
            .GetProperties(BindingFlags.Public | BindingFlags.Instance)
            .ToList();

        var queryPropertyNames = queryPropertyTypes.ConvertAll(o => o.Name);


        var orderQueryBuilder = new StringBuilder();

        foreach (var parameter in orderParameters)
        {
            if (string.IsNullOrWhiteSpace(parameter))
            {
                continue;
            }
            
            var queryPropertyName = 
                parameter[0] == '-' ||
                parameter[0] == '+'
                    ? parameter.Substring(1)
                    : parameter;

            var propertyName = queryPropertyNames
                .FirstOrDefault(pn => 
                    pn.Equals(
                        queryPropertyName,
                        StringComparison.InvariantCultureIgnoreCase));

            if (propertyName == null)
            {
                continue;
            }
            
            var sortingOrder = 
                parameter[0] == '-' 
                    ? "descending" 
                    : "ascending";
            
            orderQueryBuilder.Append($"{propertyName} {sortingOrder}, ");
        }


        var orderQuery = orderQueryBuilder.ToString()
            .TrimEnd(',', ' ');

        if (String.IsNullOrWhiteSpace(orderQuery))
        {
            return entities;
        }

        return entities.OrderBy(orderQuery);
    }
}
