using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Domain.Entities;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Countries.Commands.AddCountry;

public class AddCountryCommandHandler : 
    IRequestHandler<AddCountryCommand, CountryDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public AddCountryCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<CountryDto> Handle(
        AddCountryCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Name == request.Name, cancellationToken);

        if (entity != null)
        {
            throw new DuplicateEntityException(
                "Country with given name already exists.");
        }

        entity = new Country()
        {
            Name = request.Name
        };

        entity = await _unitOfWork.CountryRepository.AddOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<CountryDto>(entity);
    }
}
