using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Countries.Commands.DeleteCountry;

public class DeleteCountryCommandHandler : IRequestHandler<DeleteCountryCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public DeleteCountryCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        DeleteCountryCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        await _unitOfWork.CountryRepository.DeleteOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
