using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using AutoMapper;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Countries.Commands.UpdateCountry;

public class UpdateCountryCommandHandler : 
    IRequestHandler<UpdateCountryCommand, CountryDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public UpdateCountryCommandHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<CountryDto> Handle(
        UpdateCountryCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Name == request.Name, cancellationToken);

        if (entity != null)
        {
            throw new DuplicateEntityException();
        }

        entity = await _unitOfWork.CountryRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        entity.Name = request.Name;

        entity = await _unitOfWork.CountryRepository.UpdateOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();

        return _mapper.Map<CountryDto>(entity);
    }
}
