using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Employees.Queries.GetEmployee;

public class GetEmployeeQueryHandler :
    IRequestHandler<GetEmployeeQuery, EmployeeDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetEmployeeQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<EmployeeDto> Handle(
        GetEmployeeQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.EmployeeRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.Documents,
            cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }


        // Hydrate employee
        
        var company = await _unitOfWork.CompanyRepository.GetOneAsync(
            e => e.Id == entity.CompanyId, cancellationToken);

        entity.Company = company;

        var account = await _unitOfWork.AccountRepository.GetOneAsync(
            e => e.Id == entity.AccountId, cancellationToken);

        entity.Account = account;


        _unitOfWork.Dispose();

        return _mapper.Map<EmployeeDto>(entity);
    }
}
