using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application
    .Identity.Accounts.Commands.AddAccount;

public class AddAccountCommandAuthorizer :
    AbstractRequestAuthorizer<AddAccountCommand>
{
    private readonly SessionUserService _sessionAccountService;

    public AddAccountCommandAuthorizer(SessionUserService sessionAccountService)
    {
        _sessionAccountService = sessionAccountService;
    }

    public override void BuildPolicy(AddAccountCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionAccountService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionAccountService.Roles
        });
    }
}
