using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Identity
    .Accounts.Commands.DeleteAccount;

public class DeleteAccountCommandAuthorizer :
    AbstractRequestAuthorizer<DeleteAccountCommand>
{
    private readonly SessionUserService _sessionUserService;

    public DeleteAccountCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(DeleteAccountCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
