using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Identity.Accounts.Commands.DeleteAccount;

public class DeleteAccountCommandHandler : IRequestHandler<DeleteAccountCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public DeleteAccountCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        DeleteAccountCommand request,
        CancellationToken cancellationToken)
    {
        var account = await _unitOfWork.AccountRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (account == null)
        {
            throw new NotFoundException();
        }

        await _unitOfWork.AccountRepository.DeleteOneAsync(
            account, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
