using cuqmbr.TravelGuide.Application.Common.Authorization;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using MediatR.Behaviors.Authorization;

namespace cuqmbr.TravelGuide.Application.Identity.Accounts.Commands.UpdateAccount;

public class UpdateAccountCommandAuthorizer :
    AbstractRequestAuthorizer<UpdateAccountCommand>
{
    private readonly SessionUserService _sessionUserService;

    public UpdateAccountCommandAuthorizer(SessionUserService sessionUserService)
    {
        _sessionUserService = sessionUserService;
    }

    public override void BuildPolicy(UpdateAccountCommand request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated= _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            RequiredRoles = [IdentityRole.Administrator],
            UserRoles = _sessionUserService.Roles
        });
    }
}
