using cuqmbr.TravelGuide.Application.Common.FluentValidation;
using cuqmbr.TravelGuide.Application.Common.Services;
using cuqmbr.TravelGuide.Domain.Enums;
using FluentValidation;
using Microsoft.Extensions.Localization;

namespace cuqmbr.TravelGuide.Application
    .Identity.Accounts.Commands.UpdateAccount;

public class UpdateAccountCommandValidator :
    AbstractValidator<UpdateAccountCommand>
{
    public UpdateAccountCommandValidator(
        IStringLocalizer localizer,
        SessionCultureService cultureService)
    {
        RuleFor(v => v.Guid)
            .NotEmpty()
            .WithMessage(localizer["FluentValidation.NotEmpty"]);

        RuleFor(v => v.Username)
            .MinimumLength(1)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MinimumLength"],
                    1))
            .MaximumLength(32)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MaximumLength"],
                    32))
            .IsUsername()
            .WithMessage(localizer["FluentValidation.IsUsername"]);

        RuleFor(v => v.Email)
            .IsEmail()
            .WithMessage(localizer["FluentValidation.IsEmail"]);

        RuleFor(v => v.Password)
            .MinimumLength(8)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MinimumLength"],
                    8))
            .MaximumLength(64)
            .WithMessage(
                String.Format(
                    cultureService.Culture,
                    localizer["FluentValidation.MaximumLength"],
                    64));

        RuleFor(v => v.Roles ?? new IdentityRole[0])
            .IsUnique(r => r)
            .WithMessage(localizer["FluentValidation.IsUnique"]);

        RuleForEach(v => v.Roles)
            .Must(r => IdentityRole.Enumerations.ContainsValue(r))
            .WithMessage(
                String.Format(
                    localizer["FluentValidation.MustBeInEnum"],
                    String.Join(
                        ", ",
                        IdentityRole.Enumerations.Values.Select(e => e.Name))));
    }
}
