using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;

namespace cuqmbr.TravelGuide.Application.Identity.Accounts.Queries.GetAccount;

public class GetAccountQueryHandler :
    IRequestHandler<GetAccountQuery, AccountDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetAccountQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<AccountDto> Handle(
        GetAccountQuery request,
        CancellationToken cancellationToken)
    {
        var account = await _unitOfWork.AccountRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.AccountRoles,
            cancellationToken);

        if (account == null)
        {
            throw new NotFoundException();
        }


        var accountRoleIds = account.AccountRoles.Select(ar => ar.RoleId);
        var accountRoles = (await _unitOfWork.AccountRoleRepository
            .GetPageAsync(
                ar => accountRoleIds.Contains(ar.RoleId),
                ar => ar.Role,
                1, accountRoleIds.Count(), cancellationToken))
            .Items;

        account.AccountRoles = accountRoles.ToList();


        _unitOfWork.Dispose();

        return _mapper.Map<AccountDto>(account);
    }
}
