using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.Regions.Commands.DeleteRegion;

public class DeleteRegionCommandHandler : IRequestHandler<DeleteRegionCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public DeleteRegionCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        DeleteRegionCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.RegionRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        await _unitOfWork.RegionRepository.DeleteOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
