using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;
using AutoMapper;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Routes.Queries.GetRoute;

public class GetRouteQueryHandler :
    IRequestHandler<GetRouteQuery, RouteDto>
{
    private readonly UnitOfWork _unitOfWork;
    private readonly IMapper _mapper;

    public GetRouteQueryHandler(
        UnitOfWork unitOfWork,
        IMapper mapper)
    {
        _unitOfWork = unitOfWork;
        _mapper = mapper;
    }

    public async Task<RouteDto> Handle(
        GetRouteQuery request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.RouteRepository.GetOneAsync(
            e => e.Guid == request.Guid, e => e.RouteAddresses,
            cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        // TODO: Find a way to include through lists
        var addresses = await _unitOfWork.AddressRepository.GetPageAsync(
            e => entity.RouteAddresses.Select(ra => ra.AddressId).Contains(e.Id),
            e => e.City.Region.Country,
            1, entity.RouteAddresses.Count, cancellationToken);

        entity.RouteAddresses = entity.RouteAddresses.Select(
            e => new RouteAddress()
            {
                Id = e.Id,
                Guid = e.Guid,
                Order = e.Order,
                RouteId = e.RouteId,
                Route = e.Route,
                AddressId = e.AddressId,
                Address = addresses.Items.First(a => a.Id == e.AddressId)
            })
            .OrderBy(e => e.Order)
            .ToArray();

        _unitOfWork.Dispose();

        return _mapper.Map<RouteDto>(entity);
    }
}
