using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.Routes;

public sealed class RouteAddressDto : IMapFrom<RouteAddress>
{
    public Guid RouteAddressUuid { get; set; }

    public short Order { get; set; }


    public Guid Uuid { get; set; }

    public string Name { get; set; }

    public double Longitude { get; set; }

    public double Latitude { get; set; }

    public string VehicleType { get; set; }

    public Guid CountryUuid { get; set; }

    public string CountryName { get; set; }

    public Guid RegionUuid { get; set; }

    public string RegionName { get; set; }

    public Guid CityUuid { get; set; }

    public string CityName { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<RouteAddress, RouteAddressDto>()
            .ForMember(
                d => d.RouteAddressUuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Address.Guid))
            .ForMember(
                d => d.Name,
                opt => opt.MapFrom(s => s.Address.Name))
            .ForMember(
                d => d.Longitude,
                opt => opt.MapFrom(s => s.Address.Longitude))
            .ForMember(
                d => d.Latitude,
                opt => opt.MapFrom(s => s.Address.Latitude))
            .ForMember(
                d => d.VehicleType,
                opt => opt.MapFrom(s => s.Address.VehicleType.Name))
            .ForMember(
                d => d.CityUuid,
                opt => opt.MapFrom(s => s.Address.City.Guid))
            .ForMember(
                d => d.CityName,
                opt => opt.MapFrom(s => s.Address.City.Name))
            .ForMember(
                d => d.RegionUuid,
                opt => opt.MapFrom(s => s.Address.City.Region.Guid))
            .ForMember(
                d => d.RegionName,
                opt => opt.MapFrom(s => s.Address.City.Region.Name))
            .ForMember(
                d => d.CountryUuid,
                opt => opt.MapFrom(s => s.Address.City.Region.Country.Guid))
            .ForMember(
                d => d.CountryName,
                opt => opt.MapFrom(s => s.Address.City.Region.Country.Name));
    }
}
