using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Domain.Enums;

namespace cuqmbr.TravelGuide.Application.TicketGroups
    .Commands.RemoveOldReservedTicketGroups;

public class RemoveOldReservedTicketGroupsCommandHandler :
    IRequestHandler<RemoveOldReservedTicketGroupsCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public RemoveOldReservedTicketGroupsCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        RemoveOldReservedTicketGroupsCommand request,
        CancellationToken cancellationToken)
    {
        var statusesToRemove = new TicketStatus[]
        {
            TicketStatus.Reserved
        };

        var entities = (await _unitOfWork.TicketGroupRepository
            .GetPageAsync(
                e =>
                    statusesToRemove.Contains(e.Status) &&
                    DateTimeOffset.UtcNow - e.PurchaseTime > request.ReservedFor,
                1, int.MaxValue, cancellationToken))
            .Items;

        foreach (var entity in entities)
        {
            await _unitOfWork.TicketGroupRepository
                .DeleteOneAsync(entity, cancellationToken);
        }

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
