using cuqmbr.TravelGuide.Application.Common.Mappings;
using cuqmbr.TravelGuide.Application.Common.Mappings.Resolvers;
using cuqmbr.TravelGuide.Domain.Entities;

namespace cuqmbr.TravelGuide.Application.TicketGroups;

public sealed class TicketGroupDto : IMapFrom<TicketGroup>
{
    public Guid Uuid { get; set; }

    public string PassangerFirstName { get; set; }

    public string PassangerLastName { get; set; }

    public string PassangerPatronymic { get; set; }

    public string PassangerSex { get; set; }

    public DateOnly PassangerBirthDate { get; set; }

    public DateTimeOffset PurchaseTime { get; set; }

    public bool Returned { get; set; }

    public TimeSpan TravelTime { get; set; }


    public ICollection<TicketDto> Tickets { get; set; }

    public void Mapping(MappingProfile profile)
    {
        profile.CreateMap<TicketGroup, TicketGroupDto>()
            .ForMember(
                d => d.Uuid,
                opt => opt.MapFrom(s => s.Guid))
            .ForMember(
                d => d.PassangerSex,
                opt => opt.MapFrom(s => s.PassangerSex.Name))
            .ForMember(
                d => d.PurchaseTime,
                opt => opt
                    .MapFrom<DateTimeOffsetToLocalResolver, DateTimeOffset>(
                        s => s.PurchaseTime));
    }
}
