using MediatR;
using cuqmbr.TravelGuide.Application.Common.Persistence;
using cuqmbr.TravelGuide.Application.Common.Exceptions;

namespace cuqmbr.TravelGuide.Application.VehicleEnrollments.Commands.DeleteVehicleEnrollment;

public class DeleteVehicleEnrollmentCommandHandler :
    IRequestHandler<DeleteVehicleEnrollmentCommand>
{
    private readonly UnitOfWork _unitOfWork;

    public DeleteVehicleEnrollmentCommandHandler(UnitOfWork unitOfWork)
    {
        _unitOfWork = unitOfWork;
    }

    public async Task Handle(
        DeleteVehicleEnrollmentCommand request,
        CancellationToken cancellationToken)
    {
        var entity = await _unitOfWork.VehicleEnrollmentRepository.GetOneAsync(
            e => e.Guid == request.Guid, cancellationToken);

        if (entity == null)
        {
            throw new NotFoundException();
        }

        // TODO: Check for tickets bought for this enrollment.
        // Decide whether to cancel tickets or do not allow deletion.

        await _unitOfWork.VehicleEnrollmentRepository.DeleteOneAsync(
            entity, cancellationToken);

        await _unitOfWork.SaveAsync(cancellationToken);
        _unitOfWork.Dispose();
    }
}
