using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using PersistenceConfigurationOptions =
    cuqmbr.TravelGuide.Persistence.ConfigurationOptions;
using ApplicationConfigurationOptions =
    cuqmbr.TravelGuide.Application.ConfigurationOptions;
using InfrastructureConfigurationOptions =
    cuqmbr.TravelGuide.Infrastructure.ConfigurationOptions;

namespace cuqmbr.TravelGuide.Configuration.Configuration;

public static class Configuration
{
    public static IServiceCollection ConfigureConfiguration(
        this IServiceCollection services,
        string[] args)
    {
        var environment = 
            Environment.GetEnvironmentVariable("TravelGuide_Environment");

        var configuration = new ConfigurationBuilder()
            .AddJsonFile($"./appsettings.{environment}.json", optional: true)
            .AddJsonFile($"./appsettings.json", optional: true)
            .AddEnvironmentVariables(prefix: "TravelGuide_")
            .AddCommandLine(args)
            .Build();

        services.AddOptions<PersistenceConfigurationOptions>()
            .Bind(configuration);

        services.AddOptions<ApplicationConfigurationOptions>()
            .Bind(configuration);

        services.AddOptions<InfrastructureConfigurationOptions>()
            .Bind(configuration);

        return services;
    }
}
