using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Localization;
using cuqmbr.TravelGuide.Application.Common.Services;

namespace cuqmbr.TravelGuide.HttpApi.Controllers;

[Route("tests")]
public class TestsController : ControllerBase
{
    private readonly IStringLocalizer _localizer;
    private readonly EmailSenderService _emailSender;

    public TestsController(SessionCultureService cultureService,
        IStringLocalizer localizer, EmailSenderService emailSender)
    {
        _localizer = localizer;
        _emailSender = emailSender;
    }

    [HttpGet("getLocalizedString/{inputString}")]
    public Task<string> GetLocalizedString(
        [FromRoute] string inputString,
        CancellationToken cancellationToken)
    {
        return Task.FromResult<string>(_localizer[inputString]);
    }

    [HttpGet("trigger")]
    public async Task Trigger(CancellationToken cancellationToken)
    {
        var body =
@"Hello, friend!

This is my email message for you.

--
Travel Guide Service
";

        await _emailSender.SendAsync(new string[] { "cuqmbr@ya.ru" }, "Test subject", body, cancellationToken);
    }
}
