using cuqmbr.TravelGuide.Application.Common.Services;
using MailKit.Net.Smtp;
using Microsoft.Extensions.Options;
using MimeKit;

namespace cuqmbr.TravelGuide.Infrastructure.Services;

public sealed class MailKitEmailSenderService : EmailSenderService
{

    private readonly SmtpConfigurationOptions _configuration;

    public MailKitEmailSenderService(
        IOptions<ConfigurationOptions> configuration)
    {
        _configuration = configuration.Value.Email.Smtp;
    }

    public async Task SendAsync(string[] addresses, string subject,
        string body, CancellationToken cancellationToken)
    {
        var message = new MimeMessage();

        message.From.Add(new MailboxAddress(
            _configuration.SenderName, _configuration.SenderAddress));
        foreach (var address in addresses)
        {
            message.To.Add(new MailboxAddress("", address));
        }
        message.Subject = subject;

        message.Body = new TextPart("plain")
        {
            Text = body
        };


        using var client = new SmtpClient();

        await client.ConnectAsync(_configuration.Host,
            _configuration.Port, _configuration.UseTls,
            cancellationToken);

        await client.AuthenticateAsync(_configuration.Username,
            _configuration.Password, cancellationToken);

        await client.SendAsync(message, cancellationToken);
        await client.DisconnectAsync(true, cancellationToken);
    }
}
