using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class AccountRoleConfiguration : BaseConfiguration<AccountRole>
{
    public override void Configure(EntityTypeBuilder<AccountRole> builder)
    {
        builder
            .ToTable("account_roles");

        base.Configure(builder);


        builder
            .Property(ar => ar.AccountId)
            .HasColumnName("account_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(ar => ar.Account)
            .WithMany(a => a.AccountRoles)
            .HasForeignKey(ar => ar.AccountId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ar => ar.AccountId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(ar => ar.AccountId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ar => ar.AccountId).Metadata.GetColumnName()}");


        builder
            .Property(ar => ar.RoleId)
            .HasColumnName("role_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(ar => ar.Role)
            .WithMany(r => r.AccountRoles)
            .HasForeignKey(ar => ar.RoleId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ar => ar.RoleId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(ar => ar.RoleId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ar => ar.RoleId).Metadata.GetColumnName()}");
    }
}
