using cuqmbr.TravelGuide.Domain.Entities;
using cuqmbr.TravelGuide.Domain.Enums;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class CompanyConfiguration : BaseConfiguration<Company>
{
    public override void Configure(EntityTypeBuilder<Company> builder)
    {
        builder
            .ToTable("companies");

        base.Configure(builder);


        builder
            .Property(c => c.Name)
            .HasColumnName("name")
            .HasColumnType("varchar(64)")
            .IsRequired(true);

        builder
            .Property(c => c.LegalAddress)
            .HasColumnName("legal_address")
            .HasColumnType("varchar(256)")
            .IsRequired(true);

        builder
            .Property(c => c.ContactEmail)
            .HasColumnName("contact_email")
            .HasColumnType("varchar(256)")
            .IsRequired(true);

        builder
            .Property(c => c.ContactPhoneNumber)
            .HasColumnName("contact_phone_number")
            .HasColumnType("varchar(64)")
            .IsRequired(true);


        builder
            .Property(c => c.AccountId)
            .HasColumnName("account_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(c => c.Account)
            .WithOne(a => a.Company)
            .HasForeignKey<Company>(c => c.AccountId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(c => c.AccountId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(c => c.AccountId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(c => c.AccountId).Metadata.GetColumnName()}");
    }
}
