using cuqmbr.TravelGuide.Domain.Entities;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class RefreshTokenConfiguration : BaseConfiguration<RefreshToken>
{
    public override void Configure(EntityTypeBuilder<RefreshToken> builder)
    {
        builder
            .ToTable("refresh_tokens");

        base.Configure(builder);


        // Base64 encoded 128 bits 
        // 128 / 8 / 3 = 5.(3) => 6 in chars \w padding => 24 out chars
        builder
            .Property(rt => rt.Value)
            .HasColumnName("value")
            .HasColumnType("varchar(24)")
            .IsRequired(true);

        builder
            .Property(rt => rt.CreationTime)
            .HasColumnName("creation_time")
            .HasColumnType("timestamptz")
            .IsRequired(true);

        builder
            .Property(rt => rt.ExpirationTime)
            .HasColumnName("expiration_time")
            .HasColumnType("timestamptz")
            .IsRequired(true);

        builder
            .Property(rt => rt.RevocationTime)
            .HasColumnName("revocation_time")
            .HasColumnType("timestamptz")
            .IsRequired(false);


        builder
            .Property(rt => rt.AccountId)
            .HasColumnName("account_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(rt => rt.Account)
            .WithMany(rt => rt.RefreshTokens)
            .HasForeignKey(rt => rt.AccountId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(rt => rt.AccountId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(rt => rt.AccountId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(rt => rt.AccountId).Metadata.GetColumnName()}");
    }
}
