using cuqmbr.TravelGuide.Domain.Entities;
using cuqmbr.TravelGuide.Domain.Enums;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Builders;

namespace cuqmbr.TravelGuide.Persistence.PostgreSql.Configurations;

public class VehicleEnrollmentConfiguration : BaseConfiguration<VehicleEnrollment>
{
    public override void Configure(EntityTypeBuilder<VehicleEnrollment> builder)
    {
        builder
            .Property(ve => ve.Currency)
            .HasColumnName("currency")
            .HasColumnType("varchar(8)")
            .IsRequired(true);

        builder
            .ToTable(
                "vehicle_enrollments",
                ve => ve.HasCheckConstraint(
                    "ck_" +
                    $"{builder.Metadata.GetTableName()}_" +
                    $"{builder.Property(ve => ve.Currency)
                        .Metadata.GetColumnName()}",
                    $"{builder.Property(ve => ve.Currency)
                        .Metadata.GetColumnName()} IN ('{String
                        .Join("', '", Currency.Enumerations
                            .Values.Select(v => v.Name))}')"));

        base.Configure(builder);


        builder
            .Property(ve => ve.DepartureTime)
            .HasColumnName("departure_time")
            .HasColumnType("timestamptz")
            .IsRequired(true);


        builder
            .Property(ve => ve.VehicleId)
            .HasColumnName("vehicle_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(ve => ve.Vehicle)
            .WithMany(v => v.Enrollments)
            .HasForeignKey(ve => ve.VehicleId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ve => ve.VehicleId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(ve => ve.VehicleId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ve => ve.VehicleId).Metadata.GetColumnName()}");


        builder
            .Property(ve => ve.RouteId)
            .HasColumnName("route_id")
            .HasColumnType("bigint")
            .IsRequired(true);

        builder
            .HasOne(ve => ve.Route)
            .WithMany(r => r.VehicleEnrollments)
            .HasForeignKey(ve => ve.RouteId)
            .HasConstraintName(
                "fk_" + 
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ve => ve.RouteId).Metadata.GetColumnName()}")
            .OnDelete(DeleteBehavior.Cascade);

        builder
            .HasIndex(ve => ve.RouteId)
            .HasDatabaseName(
                "ix_" +
                $"{builder.Metadata.GetTableName()}_" +
                $"{builder.Property(ve => ve.RouteId).Metadata.GetColumnName()}");
    }
}
