﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

#nullable disable

namespace Persistence.PostgreSql.Migrations
{
    /// <inheritdoc />
    public partial class Add_Countries_Regions_Cities_Addresses : Migration
    {
        /// <inheritdoc />
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.EnsureSchema(
                name: "application");

            migrationBuilder.CreateSequence(
                name: "addresses_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "cities_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "countries_id_sequence",
                schema: "application");

            migrationBuilder.CreateSequence(
                name: "regions_id_sequence",
                schema: "application");

            migrationBuilder.CreateTable(
                name: "countries",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.countries_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_countries", x => x.id);
                    table.UniqueConstraint("altk_countries_Guid", x => x.uuid);
                });

            migrationBuilder.CreateTable(
                name: "regions",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.regions_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    country_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_regions", x => x.id);
                    table.UniqueConstraint("altk_regions_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_regions_country_id",
                        column: x => x.country_id,
                        principalSchema: "application",
                        principalTable: "countries",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "cities",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.cities_id_sequence')"),
                    name = table.Column<string>(type: "varchar(64)", nullable: false),
                    region_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_cities", x => x.id);
                    table.UniqueConstraint("altk_cities_Guid", x => x.uuid);
                    table.ForeignKey(
                        name: "fk_cities_region_id",
                        column: x => x.region_id,
                        principalSchema: "application",
                        principalTable: "regions",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "addresses",
                schema: "application",
                columns: table => new
                {
                    id = table.Column<long>(type: "bigint", nullable: false, defaultValueSql: "nextval('application.addresses_id_sequence')"),
                    name = table.Column<string>(type: "varchar(128)", nullable: false),
                    Longitude = table.Column<double>(type: "double precision", nullable: false),
                    Latitude = table.Column<double>(type: "double precision", nullable: false),
                    vehicle_type = table.Column<string>(type: "varchar(16)", nullable: false),
                    city_id = table.Column<long>(type: "bigint", nullable: false),
                    uuid = table.Column<Guid>(type: "uuid", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("pk_addresses", x => x.id);
                    table.UniqueConstraint("altk_addresses_Guid", x => x.uuid);
                    table.CheckConstraint("ck_addresses_vehicle_type", "vehicle_type IN ('bus', 'train', 'aircraft')");
                    table.ForeignKey(
                        name: "fk_addresses_city_id",
                        column: x => x.city_id,
                        principalSchema: "application",
                        principalTable: "cities",
                        principalColumn: "id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "ix_addresses_city_id",
                schema: "application",
                table: "addresses",
                column: "city_id");

            migrationBuilder.CreateIndex(
                name: "ix_addresses_id",
                schema: "application",
                table: "addresses",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_addresses_uuid",
                schema: "application",
                table: "addresses",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_cities_id",
                schema: "application",
                table: "cities",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_cities_region_id",
                schema: "application",
                table: "cities",
                column: "region_id");

            migrationBuilder.CreateIndex(
                name: "ix_cities_uuid",
                schema: "application",
                table: "cities",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_countries_id",
                schema: "application",
                table: "countries",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_countries_uuid",
                schema: "application",
                table: "countries",
                column: "uuid",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_regions_country_id",
                schema: "application",
                table: "regions",
                column: "country_id");

            migrationBuilder.CreateIndex(
                name: "ix_regions_id",
                schema: "application",
                table: "regions",
                column: "id",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "ix_regions_uuid",
                schema: "application",
                table: "regions",
                column: "uuid",
                unique: true);
        }

        /// <inheritdoc />
        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "addresses",
                schema: "application");

            migrationBuilder.DropTable(
                name: "cities",
                schema: "application");

            migrationBuilder.DropTable(
                name: "regions",
                schema: "application");

            migrationBuilder.DropTable(
                name: "countries",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "addresses_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "cities_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "countries_id_sequence",
                schema: "application");

            migrationBuilder.DropSequence(
                name: "regions_id_sequence",
                schema: "application");
        }
    }
}
